@@ -19,6 +19,7 @@ import tkinter.ttk as ttk
 from tkinter import filedialog
 from tkinter import messagebox
 from tkinter import scrolledtext
+from tkinter import font
 import matplotlib
 import pkg_resources
 import datetime
@@ -36,6 +37,9 @@ import numpy as np
 
 support_dir = 'models/support/'
 SQRT3 = math.sqrt(3.0)
+fontchoice=('Segoe UI', 16)
+monochoice=('Courier', fontchoice[1])
+boldchoice=(fontchoice[0], fontchoice[1], 'bold')
 
 # TODO: factor these into a separate file
 feederChoices = {
@@ -62,42 +66,27 @@ inverterChoices = {
   'VOLT_WATT':{'v':[0.90,1.06,1.10],
                'p':[1.00,1.00,0.20],
                'q':[0.00,0.00,0.00]}, 
-  'VOLT_VAR':{'v':[0.90,0.92,0.98,1.02,1.08,1.10],
-              'p':[1.00,1.00,1.00,1.00,1.00,1.00],
-              'q':[0.44,0.44,0.00,0.00,-.44,-.44]},
+  'VOLT_VAR_CATA':{'v':[0.90,1.10],
+                   'p':[1.00,1.00],
+                   'q':[0.25,-.25]},
+  'VOLT_VAR_CATB':{'v':[0.90,0.92,0.98,1.02,1.08,1.10],
+                   'p':[1.00,1.00,1.00,1.00,1.00,1.00],
+                   'q':[0.44,0.44,0.00,0.00,-.44,-.44]},
   'VOLT_VAR_AVR':{'v':[0.90,0.98,1.02,1.10],
                   'p':[1.00,1.00,1.00,1.00],
                   'q':[0.44,0.44,-.44,-.44]}, 
-  'VOLT_VAR_VOLT_WATT':{'v':[0.90,0.92,0.98,1.02,1.06,1.10],
-                        'p':[1.00,1.00,1.00,1.00,1.00,0.00],
-                        'q':[0.44,0.44,0.00,0.00,-.44,-.44]}
+  'VOLT_VAR_VOLT_WATT':{'v':[0.90,0.92,0.98,1.02,1.06,1.08,1.10],
+                        'p':[1.00,1.00,1.00,1.00,1.00,0.60,0.20],
+                        'q':[0.44,0.44,0.00,0.00,-.2933,-.44,-.44]},
+  'VOLT_VAR_14H':{'v':[0.90,0.94,0.97,1.03,1.06,1.10],
+                  'p':[1.00,1.00,1.00,1.00,1.00,0.00],
+                  'q':[0.44,0.44,0.00,0.00,-.44,-.44]}
   }
 
-solutionModeChoices = ['SNAPSHOT', 'DAILY', 'DUTY', 'YEARLY']
-controlModeChoices = ['OFF', 'STATIC', 'TIME', 'EVENT']
-
-# var columns are label, value, hint, JSON class, JSON attribute
-# if there is a sixth column, that will be the name of a Choices tuple, to be edited via Combobox
-# if there is not a sixth column, that indicates a single value to be edited via Entry
+solutionModeChoices = ['SNAPSHOT', 'DAILY', 'DUTY']#, 'YEARLY']
+controlModeChoices = ['OFF', 'STATIC'] #, 'TIME', 'EVENT']
 
 class DERConfigGUI:
-  """Manages a seven-page GUI for case configuration
-
-  The GUI opens and saves a JSON file in the format used by *tesp.tesp_config*
-
-  Todo:
-    * Possible data loss if the user updated the number of Monte Carlo cases, but didn't click the Update button before saving the case configuration.
-
-  Attributes:
-    nb (Notebook): the top-level GUI with tabbed pages
-    f1 (Frame): the page for feeder selection
-    f2 (Frame): the page for DER placement
-    f3 (Frame): the page for solar profile selection
-    f4 (Frame): the page for load profile selection
-    f5 (Frame): the page for inverter profile selection
-    f6 (Frame): the page for time-scheduled thermostat settings
-  """
-
   def __init__(self, master):
     self.master = master
     self.master.protocol('WM_DELETE_WINDOW', self.on_closing)
@@ -105,8 +94,10 @@ class DERConfigGUI:
     self.nb = ttk.Notebook(master)
     self.nb.pack(fill='both', expand='yes')
 
+    s = ttk.Style()
+    s.configure('.', font=fontchoice)
+
     for key, row in solarChoices.items():
-#      fname = os.path.join (support_dir, row['file'])
       fname = pkg_resources.resource_filename (__name__, support_dir + row['file'])
       row['data'] = np.loadtxt (fname)
       row['npts'] = row['data'].shape[0]
@@ -116,7 +107,7 @@ class DERConfigGUI:
     self.f1 = ttk.Frame(self.nb, name='varsNet')
     lab = ttk.Label(self.f1, text='Feeder Model: ', relief=tk.RIDGE)
     lab.grid(row=0, column=0, sticky=tk.NSEW)
-    self.cb_feeder = ttk.Combobox(self.f1, values=[i for i in feederChoices], name='cbFeeders')
+    self.cb_feeder = ttk.Combobox(self.f1, values=[i for i in feederChoices], name='cbFeeders', font=fontchoice)
     self.cb_feeder.set(next(iter(feederChoices)))
     self.cb_feeder.grid(row=0, column=1, sticky=tk.NSEW)
     self.cb_feeder.bind('<<ComboboxSelected>>', self.UpdateFeeder)
@@ -145,7 +136,7 @@ class DERConfigGUI:
     self.f3 = ttk.Frame(self.nb, name='varsSolar')
     lab = ttk.Label(self.f3, text='Solar Profile: ', relief=tk.RIDGE)
     lab.grid(row=0, column=0, sticky=tk.NSEW)
-    self.cb_solar = ttk.Combobox(self.f3, values=[i for i in solarChoices], name='cbSolars')
+    self.cb_solar = ttk.Combobox(self.f3, values=[i for i in solarChoices], name='cbSolars', font=fontchoice)
     self.cb_solar.set(next(iter(solarChoices)))
     self.cb_solar.grid(row=0, column=1, sticky=tk.NSEW)
     self.cb_solar.bind('<<ComboboxSelected>>', self.UpdateSolarProfile)
@@ -165,13 +156,13 @@ class DERConfigGUI:
     self.f4 = ttk.Frame(self.nb, name='varsLoad')
     lab = ttk.Label(self.f4, text='Load Profile: ', relief=tk.RIDGE)
     lab.grid(row=0, column=0, sticky=tk.NSEW)
-    self.cb_load = ttk.Combobox(self.f4, values=[i for i in loadChoices], name='cbLoads')
+    self.cb_load = ttk.Combobox(self.f4, values=[i for i in loadChoices], name='cbLoads', font=fontchoice)
     self.cb_load.set(next(iter(loadChoices)))
     self.cb_load.grid(row=0, column=1, sticky=tk.NSEW)
     self.cb_load.bind('<<ComboboxSelected>>', self.UpdateLoadProfile)
     lab = ttk.Label(self.f4, text='Load Multiplier: ', relief=tk.RIDGE)
     lab.grid(row=0, column=2, sticky=tk.NSEW)
-    self.ent_load_mult = ttk.Entry(self.f4, name='load_mult')
+    self.ent_load_mult = ttk.Entry(self.f4, name='load_mult', font=fontchoice)
     self.ent_load_mult.insert(0, '1.0')
     self.ent_load_mult.grid(row=0, column=3, sticky=tk.NSEW)
     self.f4.columnconfigure(0, weight=0)
@@ -192,13 +183,13 @@ class DERConfigGUI:
     self.f5 = ttk.Frame(self.nb, name='varsInverter')
     lab = ttk.Label(self.f5, text='Inverter Mode: ', relief=tk.RIDGE)
     lab.grid(row=0, column=0, sticky=tk.NSEW)
-    self.cb_inverter = ttk.Combobox(self.f5, values=[i for i in inverterChoices], name='cbInverters')
+    self.cb_inverter = ttk.Combobox(self.f5, values=[i for i in inverterChoices], name='cbInverters', font=fontchoice)
     self.cb_inverter.set(next(iter(inverterChoices)))
     self.cb_inverter.grid(row=0, column=1, sticky=tk.NSEW)
     self.cb_inverter.bind('<<ComboboxSelected>>', self.UpdateInverterMode)
     lab = ttk.Label(self.f5, text='Power Factor: ', relief=tk.RIDGE)
     lab.grid(row=0, column=2, sticky=tk.NSEW)
-    self.ent_inv_pf = ttk.Entry(self.f5, name='inv_pf')
+    self.ent_inv_pf = ttk.Entry(self.f5, name='inv_pf', font=fontchoice)
     self.ent_inv_pf.insert(0, '1.0')
     self.ent_inv_pf.grid(row=0, column=3, sticky=tk.NSEW)
     self.f5.columnconfigure(0, weight=0)
@@ -219,33 +210,35 @@ class DERConfigGUI:
     self.f6 = ttk.Frame(self.nb, name='varsOutput')
     lab = ttk.Label(self.f6, text='Solution Mode', relief=tk.RIDGE)
     lab.grid(row=0, column=0, sticky=tk.NSEW)
-    self.cb_soln_mode = ttk.Combobox(self.f6, values=solutionModeChoices, name='cbSolutionMode')
+    self.cb_soln_mode = ttk.Combobox(self.f6, values=solutionModeChoices, name='cbSolutionMode', font=fontchoice)
     self.cb_soln_mode.grid(row=0, column=1, sticky=tk.NSEW)
     self.cb_soln_mode.set('DAILY')
     lab = ttk.Label(self.f6, text='Control Mode', relief=tk.RIDGE)
     lab.grid(row=0, column=2, sticky=tk.NSEW)
-    self.cb_ctrl_mode = ttk.Combobox(self.f6, values=controlModeChoices, name='cbControlMode')
+    self.cb_ctrl_mode = ttk.Combobox(self.f6, values=controlModeChoices, name='cbControlMode', font=fontchoice)
     self.cb_ctrl_mode.grid(row=0, column=3, sticky=tk.NSEW)
     self.cb_ctrl_mode.set('STATIC')
     lab = ttk.Label(self.f6, text='Stop Time [min]:', relief=tk.RIDGE)
     lab.grid(row=1, column=0, sticky=tk.NSEW)
-    self.ent_stop = ttk.Entry(self.f6, name='stop')
+    self.ent_stop = ttk.Entry(self.f6, name='stop', font=fontchoice)
     self.ent_stop.insert(0, '1440')
     self.ent_stop.grid(row=1, column=1, sticky=tk.NSEW)
     lab = ttk.Label(self.f6, text='Time Step [s]:', relief=tk.RIDGE)
     lab.grid(row=1, column=2, sticky=tk.NSEW)
-    self.ent_step = ttk.Entry(self.f6, name='step')
+    self.ent_step = ttk.Entry(self.f6, name='step', font=fontchoice)
     self.ent_step.insert(0, '300')
     self.ent_step.grid(row=1, column=3, sticky=tk.NSEW)
-    #ttk.Style().configure('TButton', background='blue')
     self.output_details = tk.IntVar()
     self.output_details.set(1)
-    self.cbk_detail = ttk.Checkbutton(self.f6, text='Output PV Details', variable=self.output_details)
+    self.cbk_detail = tk.Checkbutton(self.f6, text='Output PV Details', variable=self.output_details, font=fontchoice)
     self.cbk_detail.grid(row=2, column=0, sticky=tk.NSEW)
-    ttk.Style().configure('TButton', foreground='blue')
-    self.btn_run = ttk.Button(self.f6, text='Run', command=self.RunOpenDSS)
-    self.btn_run.grid(row=2, column=1, sticky=tk.NSEW)
-    self.txt_output = scrolledtext.ScrolledText(self.f6)
+    self.output_clear = tk.IntVar()
+    self.output_clear.set(1)
+    self.cbk_clear = tk.Checkbutton(self.f6, text='Clear Old Output', variable=self.output_clear, font=fontchoice)
+    self.cbk_clear.grid(row=2, column=1, sticky=tk.NSEW)
+    self.btn_run = tk.Button(self.f6, text='Run', command=self.RunOpenDSS, font=boldchoice, bg='blue', fg='white')
+    self.btn_run.grid(row=2, column=2, sticky=tk.NSEW)
+    self.txt_output = scrolledtext.ScrolledText(self.f6, font=monochoice)
     self.txt_output.grid(row=3, columnspan=4, sticky=tk.W + tk.E + tk.N + tk.S)
     self.f6.rowconfigure (0, weight=0)
     self.f6.rowconfigure (1, weight=0)
@@ -266,19 +259,25 @@ class DERConfigGUI:
   def on_closing(self):
     self.master.quit()
     self.master.destroy()
-#    if messagebox.askokcancel('Quit', 'Do you want to close this window? This is likely to stop all simulations.'):
+#    if messagebox.askokcancel('Quit', 'Do you want to close this window? It will stop all simulations.'):
 #      self.master.quit()
 #      self.master.destroy()
 
-  def CheckEntries(self, load_mult, stop_minutes, step_seconds, inv_pf):
+  def CheckEntries(self, load_mult, stop_minutes, step_seconds, inv_pf, soln_mode):
     errors = []
-    if (stop_minutes < 1) or (stop_minutes > 1440):
-      errors.append ('Stop Minutes {:d} must be greater than 0 and no more than 1440'.format (stop_minutes))
-    if (step_seconds < 1) or (step_seconds > 300):
-      errors.append ('Step Seconds {:d} must be greater than 0 and no more than 300'.format (step_seconds))
+    if soln_mode == 'DAILY':
+      if (stop_minutes < 1) or (stop_minutes > 1440):
+        errors.append ('In DAILY mode, Stop Minutes {:d} must be greater than 0 and no more than 1440'.format (stop_minutes))
+      if (step_seconds < 1) or (step_seconds > 300):
+        errors.append ('In DAILY mode, Step Seconds {:d} must be greater than 0 and no more than 300'.format (step_seconds))
+    elif soln_mode == 'DUTY':
+      if (stop_minutes < 1) or (stop_minutes > 1440):
+        errors.append ('In DUTY mode, Stop Minutes {:d} must be greater than 0 and no more than 60'.format (stop_minutes))
+      if (step_seconds < 1) or (step_seconds > 300):
+        errors.append ('In DUTY mode, Step Seconds {:d} must be greater than 0 and no more than 5'.format (step_seconds))
     if (load_mult <= 0.0) or (load_mult > 1.0):
       errors.append ('Load Multiplier {:.4f} must be greater than 0.0 and no more than 1.0'.format (load_mult))
-    if (inv_pf <= 0.0) or (inv_pf > 1.0):
+    if (abs(inv_pf) <= 0.8) or (abs(inv_pf) > 1.0):
       errors.append ('Inverter Power Factor {:.4f} must have magnitude between 0.8 and 1.0, inclusive'.format (inv_pf))
     if len(errors) > 0:
       messagebox.showerror('Please Correct these Errors', '\n'.join(errors))
@@ -311,15 +310,34 @@ class DERConfigGUI:
       large_total += kw
       kva = float(self.f2.nametowidget('kva_{:s}'.format(key)).get())
       dertype = self.f2.nametowidget('type_{:s}'.format(key)).get()
+      oldtype = row['type']
       if (kw < 0.0) or (kva < kw):
         errors.append ('{:s} {:s} kw={:.2f} kva={:.2f} must have kw>=0 and kva>=kw'.format(dertype, key, kw, kva))
         bValid = False
-      elif dertype == 'solar':
-        change_lines.append('edit pvsystem.{:s} kva={:.2f} pmpp={:.2f}'.format(key, kva, kw))
-      elif dertype == 'storage':
-        change_lines.append('edit storage.{:s} kva={:.2f} kw={:.2f}'.format(key, kva, kw))
-      elif dertype == 'generator':
-        change_lines.append('edit generator.{:s} kva={:.2f} kw={:.2f}'.format(key, kva, kw))
+      elif dertype == oldtype:  # change the size or dispatch, but not the type
+        if dertype == 'solar':
+          change_lines.append('edit pvsystem.{:s} kva={:.2f} pmpp={:.2f}'.format(key, kva, kw))
+        elif dertype == 'storage':
+          change_lines.append('edit storage.{:s} kva={:.2f} kw={:.2f}'.format(key, kva, kw))
+        elif dertype == 'generator':
+          change_lines.append('edit generator.{:s} kva={:.2f} kw={:.2f}'.format(key, kva, kw))
+      else:
+        bus = row['bus']
+        kv = row['kv']
+        # remove the old one
+        if oldtype == 'solar':
+          change_lines.append('edit pvsystem.{:s} enabled=no'.format(key))
+        elif oldtype == 'storage':
+          change_lines.append('edit storage.{:s} enabled=no'.format(key))
+        elif oldtype == 'generator':
+          change_lines.append('edit generator.{:s} enabled=no'.format(key))
+        # create a new one
+        if dertype == 'solar':
+          change_lines.append('new pvsystem.{:s} bus1={:s} kv={:.3f} kva={:.3f} pmpp={:.3f} irrad=1'.format(key, bus, kv, kva, kw))
+        elif dertype == 'storage':
+          change_lines.append('new storage.{:s} bus1={:s} kv={:.3f} kva={:.3f} kw={:.3f}'.format(key, bus, kv, kva, kw))
+        elif dertype == 'generator':
+          change_lines.append('new generator.{:s} bus1={:s} kv={:.3f} kva={:.3f} kw={:.3f}'.format(key, bus, kv, kva, kw))
 
     if len(errors) > 0:
       messagebox.showerror('Please Correct these DER Entries', '\n'.join(errors))
@@ -327,13 +345,18 @@ class DERConfigGUI:
     return bValid, large_total, rooftop_total, change_lines
 
   def get_pv_kv_base(self, name):
+    kvnom = 0.120
     if name in self.pvder:
       row = self.pvder[name]
       kvnom = row['kv']
       if row['phases'] > 1:
         kvnom /= SQRT3
-      return kvnom
-    return 0.120
+    elif name in self.largeder: # maybe user changed the type from generator or storage
+      row = self.largeder[name]
+      kvnom = row['kv']
+      if row['phases'] > 1:
+        kvnom /= SQRT3
+    return kvnom
 
   def RunOpenDSS(self):
     load_mult = float(self.ent_load_mult.get())
@@ -347,25 +370,28 @@ class DERConfigGUI:
     stop_minutes = int(self.ent_stop.get())
     step_seconds = int(self.ent_step.get())
     num_steps = int (60 * stop_minutes / step_seconds)
-    if not self.CheckEntries(load_mult, stop_minutes, step_seconds, inv_pf):
+    if not self.CheckEntries(load_mult, stop_minutes, step_seconds, inv_pf, soln_mode):
       return
     der_valid, large_total, rooftop_total, change_lines = self.CollectDERChanges()
     if not der_valid:
       return
+
+    if self.output_clear.get() > 0:
+      self.txt_output.delete('1.0', tk.END)
+
     print (feeder_choice, solar_profile, load_mult, load_profile, inv_mode, inv_pf,
            soln_mode, ctrl_mode, step_seconds, num_steps)
-    dict = i2x.test_opendss_interface(choice = feeder_choice,
-                                      pvcurve = solar_profile,
-                                      loadmult = load_mult,
-                                      loadcurve = load_profile,
-                                      invmode = inv_mode,
-                                      invpf = inv_pf, 
-                                      stepsize = step_seconds, 
-                                      numsteps = num_steps,
-                                      ctrlmode = ctrl_mode,
-                                      solnmode = soln_mode,
-                                      change_lines = change_lines, 
-                                      doc_fp=None)
+    dict = i2x.run_opendss(choice = feeder_choice,
+                           pvcurve = solar_profile,
+                           loadmult = load_mult,
+                           loadcurve = load_profile,
+                           invmode = inv_mode,
+                           invpf = inv_pf, 
+                           stepsize = step_seconds, 
+                           numsteps = num_steps,
+                           ctrlmode = ctrl_mode,
+                           solnmode = soln_mode,
+                           change_lines = change_lines)
     self.txt_output.insert(tk.END, 'Analysis Run on {:s} at {:s}'.format(feeder_choice, datetime.datetime.now().strftime('%a %d-%b-%Y %H:%M:%S')))
     self.txt_output.insert(tk.END, '  Large DER={:.2f} kW, Rooftop PV={:.2f} kW\n'.format(large_total, rooftop_total))
     self.txt_output.insert(tk.END, '  LoadMult={:.4f}, LoadProfle={:s}\n'.format(load_mult, load_profile))
@@ -376,6 +402,9 @@ class DERConfigGUI:
     self.txt_output.insert(tk.END, '{:d} Nodes with Low Voltage, Lowest={:.4f}pu at {:s}\n'.format(dict['num_low_voltage'], dict['vminpu'], dict['node_vmin']))
     self.txt_output.insert(tk.END, '{:d} Nodes with High Voltage, Highest={:.4f}pu at {:s}\n'.format(dict['num_high_voltage'], dict['vmaxpu'], dict['node_vmax']))
 
+    if soln_mode == 'SNAPSHOT':
+      return
+
     base = dict['kWh_Load']
     pctSource = 0.0
     pctLosses = 0.0
@@ -466,7 +495,7 @@ class DERConfigGUI:
     lab_gen.grid(row=3, column=0, sticky=tk.NSEW)
     lab_roofs = ttk.Label(self.f2, text='{:d} Available Residential Rooftops, Use[%]:'.format(len(self.resloads)), relief=tk.RIDGE)
     lab_roofs.grid(row=4, column=0, sticky=tk.NSEW)
-    self.ent_pct_roofs = ttk.Entry(self.f2, name='pct_roofs')
+    self.ent_pct_roofs = ttk.Entry(self.f2, name='pct_roofs', font=fontchoice)
     self.ent_pct_roofs.insert(0, '10.0')
     self.ent_pct_roofs.grid(row=4, column=1, sticky=tk.NSEW)
 
@@ -489,13 +518,13 @@ class DERConfigGUI:
                                                                                                           row['type']), 
                       relief=tk.RIDGE)
       lab.grid(row=idx, column=0, sticky=tk.NSEW)
-      e1 = ttk.Entry(self.f2, name='kw_{:s}'.format(key))
+      e1 = ttk.Entry(self.f2, name='kw_{:s}'.format(key), font=fontchoice)
       e1.insert(0, str(row['kw']))
       e1.grid(row=idx, column=1, sticky=tk.NSEW)
-      e2 = ttk.Entry(self.f2, name='kva_{:s}'.format(key))
+      e2 = ttk.Entry(self.f2, name='kva_{:s}'.format(key), font=fontchoice)
       e2.insert(0, str(row['kva']))
       e2.grid(row=idx, column=2, sticky=tk.NSEW)
-      cb1 = ttk.Combobox(self.f2, values=['solar','storage','generator'], name='type_{:s}'.format(key))
+      cb1 = ttk.Combobox(self.f2, values=['solar','storage','generator'], name='type_{:s}'.format(key), font=fontchoice)
       cb1.set(row['type'])
       cb1.grid(row=idx, column=3, sticky=tk.NSEW)
       idx += 1
@@ -588,6 +617,9 @@ def show_der_config():
   """
   root = tk.Tk()
   root.title('i2x Test Systems: DER Cases')
+  bigfont = font.Font(family=fontchoice[0],size=fontchoice[1])
+  root.option_add("*TCombobox*Listbox*Font", bigfont)
+#  root.option_add("*Font", bigfont)
   my_gui = DERConfigGUI(root)
   while True:
     try:
